#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Script to create and update a repository on zope instances
# Stéphan Geulette <stephan.geulette@uvcw.be>, UVCW
# From original script "recover.py" ()
#

from OFS.Application import Application
from Products.CPUtils.scripts.utils import CreateAndCallExternalMethod
from Products.CPUtils.scripts.utils import error
from Products.CPUtils.scripts.utils import trace
from Products.CPUtils.scripts.utils import treat_zopeconflines
from Products.CPUtils.scripts.utils import verbose
from utils import datetime
from zope.component import getSiteManager
from zope.component import getUtilitiesFor

import os
import socket
import sys


buildout_inst_type = None  # True for buildout, False for manual instance
tempdir = ""
now = datetime(1973, 02, 12).now()
pfolders = {}
temp_added = False


def main():
    global instdir, tempdir, buildout_inst_type, user, pwd
    try:
        from optparse import OptionParser

        parser = OptionParser()
        parser.add_option(
            "-i",
            "--infos",
            dest="infos",
            default=None,
            help="infos about instance formatted like: "
            '"instance_path;transactions_days_number;admin_user;admin_password"',
        )
        (options, args) = parser.parse_args()
        if options.infos.startswith("#"):
            sys.exit(0)
        instdir, days, user, pwd = options.infos.split(";")
    except ValueError:
        error("Problem in parameters")
        parser.print_help()
        sys.exit(1)

    instdir = instdir.rstrip("/")
    verbose("Working on instance %s" % instdir)

    # Finding the instance type (buildout or manual)
    if os.path.exists(os.path.join(instdir, "parts")):
        buildout_inst_type = True
        # verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(instdir, "etc")):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(instdir) or True:
        error("! Invalid instance path '%s' or instance type not detected" % instdir)
        sys.exit(1)

    if buildout_inst_type:
        zodbfilename = os.path.join(instdir, "parts/instance/etc/zope.conf")
        fspath = os.path.join(instdir, "var/filestorage/")
    else:
        zodbfilename = os.path.join(instdir, "etc/zope.conf")
        fspath = os.path.join(instdir, "var/")

    instance = os.path.basename(instdir)
    if not tempdir:
        tempdir = os.path.join(instdir, "temp")
    hostname = socket.gethostname()

    port = treat_zopeconflines(zodbfilename, fspath)

    trace("host='%s', inst='%s'" % (hostname, instance))

    CreateAndCallExternalMethod(
        port, user, pwd, "cputils_checkPOSKey", "utils.py", "checkPOSKey"
    )


def _checkAttributes(obj, errors, context=None):
    from ZODB.POSException import POSKeyError

    # very dumb checks for list and dict (like) attributes
    # is very slow but ensures that all attributes are checked
    for k, v in obj.__dict__.items():
        if hasattr(v, "values") and hasattr(v, "keys"):
            try:
                [repr(val) for val in v.values()]
                [repr(val) for val in v.keys()]
            except POSKeyError as ex:
                if hasattr(obj, "getPhysicalPath"):
                    path = "/".join(obj.getPhysicalPath())
                elif context:
                    path = "Utility: %s in %s" % (
                        repr(obj),
                        "/".join(context.getPhysicalPath()),
                    )
                else:
                    path = "Utility: %s" % (repr(obj))

                error("Error %s on DICT-LIKE attribute %s (%s)" % (str(ex), k, path))
                errors.append(
                    "Error %s on DICT-LIKE attribute %s (%s)" % (str(ex), k, path)
                )

        if hasattr(v, "append"):
            try:
                [val for val in v]
            except POSKeyError as ex:
                error(
                    "Error %s on LIST-LIKE attribute %s (%s)"
                    % (str(ex), k, "/".join(obj.getPhysicalPath()))
                )
                errors.append(
                    "Error %s on LIST-LIKE attribute %s (%s)"
                    % (str(ex), k, "/".join(obj.getPhysicalPath()))
                )


# ------------------------------------------------------------------------------


def _sub(master, errors):
    from ZODB.POSException import POSKeyError

    def check_utilities(sm, master, errors):
        for one in sm.utilities._subscribers:
            for interface in one.keys():
                try:
                    for util_tup in getUtilitiesFor(interface, context=master):
                        utility = util_tup[1]
                        _checkAttributes(utility, errors, context=master)
                except TypeError as msg:
                    error(
                        "Cannot get utilities for interface '%s' : %s"
                        % (str(interface), msg)
                    )

    # check site utilities
    if master.meta_type == "Plone Site":
        from Products.CMFCore.PortalObject import PortalObjectBase

        sm = PortalObjectBase.getSiteManager(master)
        check_utilities(sm, master, errors)

    if master.__class__ == Application:
        sm = getSiteManager()
        check_utilities(sm, master, errors)

    for oid in master.objectIds():
        try:
            obj = getattr(master, oid)
            trace("%s->%s" % ("/".join(master.getPhysicalPath()), obj.getId()))
            # output.append('%s->%s' % ('/'.join(master.getPhysicalPath()), obj.getId()))
            if hasattr(obj, "objectIds") and obj.getId() != "Control_Panel":
                _sub(obj, errors)

            # check catalog explicitly
            if obj.meta_type in [
                "ZCatalog",
                "Catalog",
                "Plone Catalog Tool",
            ] or hasattr(obj, "_catalog"):
                for idxid in obj._catalog.indexes.keys():
                    try:
                        index = obj._catalog.indexes.get(idxid)
                        trace(
                            "%s->INDEX: %s" % ("/".join(obj.getPhysicalPath()), idxid)
                        )
                        # output.append('%s->INDEX: %s' % ('/'.join(obj.getPhysicalPath()), idxid))

                        _checkAttributes(index, errors)

                    except POSKeyError as ex:
                        error(
                            "Error %s on INDEX %s (%s)"
                            % (str(ex), idxid, "/".join(obj.getPhysicalPath()))
                        )
                        errors.append(
                            "Error %s on INDEX %s (%s)"
                            % (str(ex), idxid, "/".join(obj.getPhysicalPath()))
                        )

                # support for lexicon
                for lexid in obj.objectIds():
                    _checkAttributes(getattr(obj, lexid), errors)

        except POSKeyError as ex:
            error(
                "Error %s on %s (%s)"
                % (str(ex), oid, "/".join(master.getPhysicalPath()))
            )
            errors.append(
                "Error %s on %s (%s)"
                % (str(ex), oid, "/".join(master.getPhysicalPath()))
            )


def check(app, errors=[]):
    # verbose("Begin of POSKey check")
    sys.setrecursionlimit(20000)
    _sub(app, errors)
    # verbose("End of POSKey check")


if __name__ == "__main__":
    # verbose("Begin of %s"%sys.argv[0])
    main()
    # verbose("End of %s"%sys.argv[0])



from Products.FCKeditor.config import RUID_URL_PATTERN

import re


IMG = '<img title="broken image" alt="broken image" src="" />'


def removeInlineURI(text):
    pattern = re.compile(r"<img[^>]*src=[\'\"]data:[^>]*/>")
    text, count = pattern.subn(IMG, text)
    return text


def convert(self, orig, data, **kwargs):
    text = orig
    # XXX PATCH START XXX
    text = removeInlineURI(text)
    # XXX PATCH END XXX
    tags_ruid, unique_ruid = self.find_ruid(text)
    if unique_ruid:
        ruid_url = self.mapRUID_URL(unique_ruid, kwargs["context"])
        for tag_ruid in tags_ruid:
            t, uid = tag_ruid.items()[0]
            if uid in ruid_url:
                text = text.replace(
                    t,
                    t.lower().replace(
                        "./%s/%s" % (RUID_URL_PATTERN, uid), ruid_url[uid]
                    ),
                )

    data.setData(text)
    return data



#!/usr/bin/python
#
"""
    This script treats dbs of a zope instance :
    * packing each db
    * backuping with repozo each fs file
"""
from Products.CPUtils.scripts.utils import packdb
from Products.CPUtils.scripts.utils import read_zopectlfile
from Products.CPUtils.utils import verbose
from utils import error
from utils import read_zopeconffile

import datetime
import os
import shutil
import sys


buildout_inst_type = None  # True for buildout, False for manual instance
zeo_type = False  # True for zeo
BACKUP_DIR = "/srv/backups/zope"

# ------------------------------------------------------------------------------


def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    os.system(cmd + " >_cmd_zr.out 2>_cmd_zr.err")
    stdout = stderr = []
    try:
        if os.path.exists("_cmd_zr.out"):
            ofile = open("_cmd_zr.out", "r")
            stdout = ofile.readlines()
            ofile.close()
            os.remove("_cmd_zr.out")
        else:
            error("File %s does not exist" % "_cmd_zr.out")
    except IOError:
        error("Cannot open %s file" % "_cmd_zr.out")
    try:
        if os.path.exists("_cmd_zr.err"):
            ifile = open("_cmd_zr.err", "r")
            stderr = ifile.readlines()
            ifile.close()
            os.remove("_cmd_zr.err")
        else:
            error("File %s does not exist" % "_cmd_zr.err")
    except IOError:
        error("Cannot open %s file" % "_cmd_zr.err")
    return (stdout, stderr)


def treat_zopeconflines(zodbfilename):
    """
        read zope configuration lines to get informations
    """
    lines = []
    read_zopeconffile(zodbfilename, lines)
    zeo_fs = {}
    if zeo_type:
        zeoconffile = os.path.join(instdir, "parts", "zeo", "etc", "zeo.conf")
        lines2 = []
        read_zopeconffile(zeoconffile, lines2)
        fsflag = False
        fsnb = 0
        for line in lines2:
            if line.startswith("<filestorage "):
                fsflag = True
                fsnb = line.split()[1].strip(" >")
                # verbose("fsnb '%s'"%fsnb)
                continue
            if fsflag and line.startswith("path "):
                fsflag = False
                fs = line.split()[1]
                fs = os.path.basename(fs)
                # verbose("fs '%s'"%fs)
                if fsnb in zeo_fs:
                    error("Filestorage number '%s' already found" % fsnb)
                else:
                    zeo_fs[fsnb] = fs
                fsflag = False
                fsnb = 0
                continue
    port = ""
    httpflag = False
    fsflag = False
    dbs = []
    dbname = ""
    for line in lines:
        # verbose("=>'%s'"%line)
        if line.startswith("<http-server>"):
            httpflag = True
            continue
        if httpflag and line.startswith("address"):
            port = line.split()[1]
            httpflag = False
            continue
        if line.startswith("<zodb_db"):
            if dbname:
                error(
                    "\tnext db found while fs not found: previous dbname '%s', current line '%s'"
                    % (dbname, line)
                )
            dbname = line.split()[1]
            dbname = dbname.rstrip(">")
            if dbname == "temporary":
                dbname = ""
        if line.startswith("storage "):  # ZEO
            fsnb = line.split()[1]
            if fsnb in zeo_fs:
                dbs.append([dbname, zeo_fs[fsnb]])
            else:
                error("Filestorage number '%s' not found in zeo.conf ?" % fsnb)
            dbname = ""
            continue
        if line.startswith("<filestorage>"):  # NOT ZEO
            fsflag = True
            continue
        if fsflag and line.startswith("path"):  # NOT ZEO
            fsflag = False
            fs = line.split()[1]
            fs = os.path.basename(fs)
            dbs.append([dbname, fs])
            dbname = ""
            continue
    verbose(
        "\tport='%s', dbs='%s'" % (port, ";".join([",".join(dbinfo) for dbinfo in dbs]))
    )

    if not port:
        error("! the port was not found in the config file '%s'" % zodbfilename)

    return (port, dbs)


# ------------------------------------------------------------------------------


def backupdb(fs, zopepath, fspath, pythonfile):
    """ call the repozo script to backup file """
    repozofilename = os.path.join(zopepath, "bin", "repozo.py")
    pythonpath = os.path.join(zopepath, "lib", "python")
    if not os.path.exists(repozofilename):
        repozofilename = os.path.join(instdir, "bin", "repozo")
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(zopepath, "utilities", "ZODBTools", "repozo.py")
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(pythonpath, "ZODB", "scripts", "repozo.py")
    backupcmd = "env PYTHONPATH=%s %s %s -Bv " % (
        pythonpath,
        pythonfile,
        repozofilename,
    )
    if options.fullbackup:
        backupcmd += "-F "
    # -B / -R : backup or recover
    # -r backupdir
    # -F : full backup
    # -f fs file
    start = datetime.now()
    fsfilename = os.path.join(fspath, fs)
    #        backupdir = os.path.join(BACKUP_DIR, os.path.basename(instdir))
    #        if not os.path.exists(backupdir):
    #            os.mkdir(backupdir)
    backupdir = os.path.join(
        BACKUP_DIR, os.path.basename(instdir), os.path.splitext(fs)[0]
    )
    if os.path.exists(backupdir) and options.fullbackup:
        for file in shutil.os.listdir(backupdir):
            shutil.os.unlink(os.path.join(backupdir, file))
            verbose(
                "\t%s deleted because we do a full backup"
                % (os.path.join(backupdir, file))
            )
    if not os.path.exists(backupdir):
        os.makedirs(backupdir)
    verbose("\tBackup of '%s' with script '%s'" % (fsfilename, repozofilename))
    cmd = backupcmd + " -r %s -f %s" % (backupdir, fsfilename)
    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        verbose("\t\tOutput/Error when backuping : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tOutput when backuping : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)
    verbose("\t\t-> elapsed time %s" % (datetime.now() - start))


# ------------------------------------------------------------------------------


def main():
    global buildout_inst_type, zeo_type, instdir
    instance_section = "instance"

    if instdir.endswith("/"):
        instdir = instdir[0:-1]
    verbose("Working on instance %s" % instdir)

    if os.path.exists(os.path.join(instdir, "parts")):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(instdir, "etc")):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(instdir) or True:
        error("! Invalid instance path '%s' or instance type not detected" % instdir)
        sys.exit(1)

    if buildout_inst_type:
        if os.path.exists(os.path.join(instdir, "bin", "zeo")):
            zeo_type = True
            verbose("\tInstance is a zeo !")
        if os.path.exists(os.path.join(instdir, "bin", "instance1")):
            instance_section = "instance1"
        zodbfilename = instdir + "/parts/%s/etc/zope.conf" % instance_section
        zopectlfilename = instdir + "/parts/%s/bin/zopectl" % instance_section
        fspath = instdir + "/var/filestorage/"
    else:
        zodbfilename = instdir + "/etc/zope.conf"
        zopectlfilename = instdir + "/bin/zopectl"
        fspath = instdir + "/var/"

    # Getting some informations in config file
    (port, dbs) = treat_zopeconflines(zodbfilename)
    (zopepath, pythonfile) = read_zopectlfile(zopectlfilename)
    # verbose("repozo path='%s'"%zopepath)

    # We remove the folder containing all instance dbs in case of a full backup
    # It's necessary to avoid keeping old files to safe disk space
    if options.fullbackup:
        backupdir = os.path.join(BACKUP_DIR, os.path.basename(instdir))
        if os.path.exists(backupdir):
            shutil.rmtree(backupdir)
            verbose("\t%s deleted when full backup" % (backupdir))
    # Treating each db
    for db in dbs:
        packdb(
            port, db[0], days, "cputils_pack_db", "CPUtils.utils", "pack_db", user, pwd
        )
        # backupdb(db[1], zopepath, fspath, pythonfile)

    for file in shutil.os.listdir(fspath):
        if file.endswith(".fs.old"):
            shutil.os.unlink(fspath + file)
            verbose("\t%s deleted" % (fspath + file))
        elif file.endswith(".fs.pack"):
            shutil.os.unlink(fspath + file)
            error("! .pack file found : pack not correctly ended")
            verbose("\t%s deleted" % (fspath + file))


# ------------------------------------------------------------------------------

try:
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option(
        "-i",
        "--infos",
        dest="infos",
        default=None,
        help="infos about instance formatted like: "
        '"instance_path;transactions_days_number;admin_user;admin_password"',
    )
    parser.add_option(
        "-F",
        "--full",
        dest="fullbackup",
        help="do a full backup, not an incremental one",
        default=False,
        action="store_true",
    )
    (options, args) = parser.parse_args()
    if options.infos.startswith("#"):
        sys.exit(0)
    instdir, days, user, pwd = options.infos.split(";")
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == "__main__":
    verbose(
        "Start of packing, backuping %s, days=%s, full_backup=%s"
        % (instdir, days, options.fullbackup)
    )
    #    sys.exit(0)
    main()
    verbose("End of packing, backuping %s" % (instdir))



#!/usr/bin/python
from __future__ import print_function
from datetime import datetime

import os
import sys


"""
    Restore of zope fs file backuped with the script pack_backup_db.py.
    This restore is done on a backup system where all instances files are copied in a RSYNCDIR dir
    Zope must be installed exactly at the same place on the main server and backup server
"""
buildout_inst_type = None  # True for buildout, False for manual instance
BACKUP_DIR = "/srv/backups/zope"
RSYNCDIR = ""  # prefix dir containing the copy of the rsync command
FSTEST = True
FSREFS = False
OVW_FS = False  # overwrite fs file when restoring if already exist
PYTHONBIN = ""


def verbose(*messages):
    print(">>", " ".join(messages))


def error(*messages):
    print("!!", (" ".join(messages)))


def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    os.system(cmd + " >_cmd_zr.out 2>_cmd_zr.err")
    stdout = stderr = []
    try:
        if os.path.exists("_cmd_zr.out"):
            ofile = open("_cmd_zr.out", "r")
            stdout = ofile.readlines()
            ofile.close()
            os.remove("_cmd_zr.out")
        else:
            error("File %s does not exist" % "_cmd_zr.out")
    except IOError:
        error("Cannot open %s file" % "_cmd_zr.out")
    try:
        if os.path.exists("_cmd_zr.err"):
            ifile = open("_cmd_zr.err", "r")
            stderr = ifile.readlines()
            ifile.close()
            os.remove("_cmd_zr.err")
        else:
            error("File %s does not exist" % "_cmd_zr.err")
    except IOError:
        error("Cannot open %s file" % "_cmd_zr.err")
    return (stdout, stderr)


def read_zopeconffile(zodbfilename, lines):
    """ read the zope conf filename and include subfile """
    try:
        zfile = open(zodbfilename, "r")
    except IOError:
        error("! Cannot open %s file" % zodbfilename)
        return
    for line in zfile.readlines():
        line = line.strip("\n\t ")
        if line.startswith("%include"):
            otherfilename = line.split()[1]
            if RSYNCDIR:
                if otherfilename.startswith("/"):
                    otherfilename = otherfilename[1:]
                otherfilename = os.path.join(RSYNCDIR, otherfilename)
            read_zopeconffile(otherfilename, lines)
            continue
        lines.append(line)


def treat_zopeconflines(zodbfilename):
    """
        read zope configuration lines to get informations
    """
    lines = []
    read_zopeconffile(zodbfilename, lines)

    port = ""
    httpflag = False
    fsflag = False
    dbs = []
    dbname = ""
    for line in lines:
        # verbose("=>'%s'"%line)
        if line.startswith("<http-server>"):
            httpflag = True
            continue
        if httpflag and line.startswith("address"):
            port = line.split()[1]
            httpflag = False
            continue
        if line.startswith("<zodb_db"):
            if dbname:
                error(
                    "\tnext db found while fs not found: previous dbname '%s', current line '%s'"
                    % (dbname, line)
                )
            dbname = line.split()[1]
            dbname = dbname.rstrip(">")
            if dbname == "temporary":
                dbname = ""
        if line.startswith("<filestorage>"):
            fsflag = True
            continue
        if fsflag and line.startswith("path"):
            fsflag = False
            fs = line.split()[1]
            fs = os.path.basename(fs)
            dbs.append([dbname, fs])
            dbname = ""
            continue
    verbose(
        "\tport='%s', dbs='%s'" % (port, ";".join([",".join(dbinfo) for dbinfo in dbs]))
    )

    if not port:
        error("! the port was not found in the config file '%s'" % zodbfilename)

    return (port, dbs)


def read_zopectlfile(zopectlfilename):
    """ read the zopectl file to find the zope path """
    try:
        zfile = open(zopectlfilename, "r")
    except IOError:
        error("! Cannot open %s file" % zopectlfilename)
        return
    for line in zfile.readlines():
        line = line.strip("\n\t ")
        if line.startswith("ZOPE_HOME"):
            zopepath = line.split("=")[1]
            zopepath = zopepath.strip("\"' ")
            return zopepath
    return ""


def read_buildoutfile(buildoutfilename):
    """ read the buildout file to find the python path """
    try:
        zfile = open(buildoutfilename, "r")
    except IOError:
        error("! Cannot open %s file" % buildoutfilename)
        return
    for line in zfile.readlines():
        line = line.strip("\n\t ")
        if line.startswith("#!"):
            zfile.close()
            return line.strip("#! ")
    return PYTHONBIN


def restoredb(fs, zopepath, fspath):
    """ call the repozo script to restore fs """
    repozofilename = os.path.join(zopepath, "bin", "repozo.py")
    pythonpath = os.path.join(zopepath, "lib", "python")
    if not os.path.exists(repozofilename):
        repozofilename = os.path.join(instdir, "bin", "repozo")
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(zopepath, "utilities", "ZODBTools", "repozo.py")
    elif not os.path.exists(repozofilename):
        repozofilename = os.path.join(pythonpath, "ZODB", "scripts", "repozo.py")
    restorecmd = "env PYTHONPATH=%s %s %s -Rv " % (
        pythonpath,
        PYTHONBIN,
        repozofilename,
    )
    # -B / -R : backup or recover
    # -r backupdir
    # -F : full backup
    # -f fs file to backup
    # -D date to restore
    # -o restored file name
    start = datetime.now()
    fsfilename = os.path.join(fspath, fs)
    backupdir = os.path.join(
        BACKUP_DIR, os.path.basename(instdir), os.path.splitext(fs)[0]
    )
    if not OVW_FS and os.path.exists(fsfilename):
        error(
            "\tfs file '%s' already exists ! Restoring in backup dir '%s' ..."
            % (fsfilename, backupdir)
        )
        fsfilename = os.path.join(backupdir, fs)
        # return
    verbose("\tRestore of '%s'" % (fsfilename))
    cmd = restorecmd + "-r %s -o %s" % (backupdir, fsfilename)
    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        verbose("\t\tOutput/Error when restoring : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tOutput when restoring : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)
    verbose("\t\t-> elapsed time %s" % (datetime.now() - start))

    if FSTEST:
        fstestfilename = os.path.join(zopepath, "bin", "fstest.py")
        if not os.path.exists(fstestfilename):
            fstestfilename = os.path.join(
                zopepath, "utilities", "ZODBTools", "fstest.py"
            )
        elif not os.path.exists(fstestfilename):
            fstestfilename = os.path.join(pythonpath, "ZODB", "scripts", "fstest.py")
        fstestcmd = "env PYTHONPATH=%s %s %s " % (pythonpath, PYTHONBIN, fstestfilename)
        # -v : print a line by transaction
        # -vv : print a line by object
        start = datetime.now()
        verbose("\tTest of '%s'" % (fsfilename))
        cmd = fstestcmd + "%s" % (fsfilename)
        verbose("\tRunning command '%s'" % cmd)
        (cmd_out, cmd_err) = runCommand(cmd)

        if cmd_err:
            error("\t\tError when testing : '%s'" % "".join(cmd_err))
        elif cmd_out:
            error("\t\tError when testing : '%s'" % "".join(cmd_out))
        else:
            verbose("\t\tAll seems OK")
        verbose("\t\t-> elapsed time %s" % (datetime.now() - start))

    if FSREFS:
        fsrefsfilename = os.path.join(zopepath, "bin", "fsrefs.py")
        if not os.path.exists(fsrefsfilename):
            fsrefsfilename = os.path.join(
                zopepath, "utilities", "ZODBTools", "fsrefs.py"
            )
        elif not os.path.exists(fsrefsfilename):
            fsrefsfilename = os.path.join(pythonpath, "ZODB", "scripts", "fsrefs.py")
        fsrefscmd = "env PYTHONPATH=%s %s %s " % (pythonpath, PYTHONBIN, fsrefsfilename)
        # -v : print a line by transaction
        # -vv : print a line by object
        start = datetime.now()
        verbose("\tTest of '%s'" % (fsfilename))
        cmd = fsrefscmd + "%s" % (fsfilename)
        verbose("\tRunning command '%s'" % cmd)
        (cmd_out, cmd_err) = runCommand(cmd)

        if cmd_err:
            verbose("\t\tError when testing : '%s'" % "".join(cmd_err))
        elif cmd_out:
            verbose("\t\tError when testing : '%s'" % "".join(cmd_out))
        verbose("\t\t-> elapsed time %s" % (datetime.now() - start))


def copyfs(fs, fspath):
    """ copy the fs file in the right place """
    start = datetime.now()
    fsfilename = os.path.join(fspath, fs)
    backupdir = os.path.join(
        BACKUP_DIR, os.path.basename(instdir), os.path.splitext(fs)[0]
    )
    backupfile = os.path.join(backupdir, fs)
    #    if not os.path.exists(backupfile):
    #        for filename in os.listdir(backupdir):
    #            filepath = os.path.join(backupdir, filename)
    #            if filepath.endswith('.fs'):
    #                backupfile = filepath
    #                break

    #    if os.path.exists(fsfilename):
    #        error("\tfs file '%s' already exists !"%(fsfilename))
    #        return
    verbose("\tCopy of '%s' to '%s'" % (backupfile, fsfilename))
    cmd = "cp %s %s" % (backupfile, fsfilename)
    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)
    if cmd_err:
        verbose("\t\tError when copying : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tError when copying : '%s'" % "".join(cmd_out))
    verbose("\t\t-> elapsed time %s" % (datetime.now() - start))


def main():
    global buildout_inst_type, PYTHONBIN

    tmp = instdir
    if tmp.endswith("/"):
        tmp = tmp[0:-1]
    verbose("Working on instance %s" % tmp)

    if os.path.exists(os.path.join(tmp, "parts")):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(tmp, "etc")):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(tmp) or True:
        error("! Invalid instance path '%s' or instance type not detected" % tmp)
        sys.exit(1)

    if buildout_inst_type:
        zodbfilename = instdir + "/parts/instance/etc/zope.conf"
        zopectlfilename = instdir + "/parts/instance/bin/zopectl"
        fspath = instdir + "/var/filestorage/"
        PYTHONBIN = read_buildoutfile(os.path.join(instdir, "bin", "buildout"))
    else:
        zodbfilename = instdir + "/etc/zope.conf"
        zopectlfilename = instdir + "/bin/zopectl"
        fspath = instdir + "/var/"

    # Getting some informations in config file
    (port, dbs) = treat_zopeconflines(zodbfilename)
    zopepath = read_zopectlfile(zopectlfilename)
    if RSYNCDIR:
        if zopepath.startswith("/"):
            zopepath = zopepath[1:]
        zopepath = os.path.join(RSYNCDIR, zopepath)
    verbose("zope path='%s'" % zopepath)

    # Treating each db
    for db in dbs:
        if RSYNCDIR:
            copyfs(db[1], fspath)
        else:
            restoredb(db[1], zopepath, fspath)


# ------------------------------------------------------------------------------

try:
    arg = sys.argv[1]
    if arg.startswith("#"):
        sys.exit(0)
    instdir, days, user, pwd = arg.split(";")
    if RSYNCDIR:
        if instdir.startswith("/"):
            instdir = instdir[1:]
        instdir = os.path.join(RSYNCDIR, instdir)
    verbose("Start of restoring dbs of %s" % (instdir))
except IndexError:
    error("No parameter found")
    sys.exit(1)
except ValueError:
    error("No enough parameters")
    sys.exit(1)

if __name__ == "__main__":
    main()
    verbose("End of restoring dbs of %s" % (instdir))



from OFS.Application import Application
from Products.CMFPlone.utils import base_hasattr
from Products.CPUtils.config import FILES_FOLDER
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
from zope.i18nmessageid import MessageFactory

import os


_ = MessageFactory("CPUtils")


def setRobots(context):
    """
        Copy a robots.txt file from files dir to portal_skins/custom
    """
    if context.readDataFile("cputils_robots.txt") is None:
        return

    portal = context.getSite()
    output = []

    # Add additional setup code here

    zopeFolder = portal.portal_skins.custom

    filename = name = "robots.txt"
    path = os.path.join(FILES_FOLDER, filename)
    fd = open(path, "r")
    data = fd.read()
    fd.close()

    # if the file exists and title starts with Remove ..., we replace the file
    if base_hasattr(zopeFolder, name):
        rf = getattr(zopeFolder, name)
        if rf.title.startswith("Remove this file in production mode!"):
            zopeFolder.manage_delObjects([name])

    if not base_hasattr(zopeFolder, name):
        output.append("Creating file '%s' in '%s'" % (name, "portal_skins/custom"))
        zopeFolder.manage_addProduct["OFSP"].manage_addFile(
            id=name,
            file=data,
            title="Remove this file in production mode! A effacer une fois le site en production!",
        )
        if name in zopeFolder.objectIds():
            output.append("File created")
        else:
            output.append("File NOT created")
    else:
        output.append(
            "File '%s' in '%s' already exists" % (name, "portal_skins/custom")
        )

    return "\n".join(output)


def install(context):
    app = context.getSite()
    count = 0
    while not isinstance(app, Application) and count < 10:
        app = app.getParentNode()
        count += 1

    if not isinstance(app, Application):
        raise ValueError("Couldn't find parent Application")

    if not hasattr(app, 'cputils_install'):
        manage_addExternalMethod(app, 'cputils_install', '', 'CPUtils.utils', 'install')
    # we run this method
    app.cputils_install(app)



from Acquisition import aq_base
from config import PLONE_VERSION
from config import product_globals
from Products.CMFCore import DirectoryView
from Products.CMFCore.utils import getToolByName
from Products.CMFQuickInstallerTool.QuickInstallerTool import QuickInstallerTool
from Products.validation.validators.SupplValidators import MaxSizeValidator
from ZPublisher.HTTPRequest import FileUpload

import logging


logger = logging.getLogger("CPUtils")
logger.info("Installing Product")

try:
    from Products.validation.i18n import PloneMessageFactory as _
    from Products.validation.i18n import recursiveTranslate
    from Products.validation.i18n import safe_unicode
except ImportError:
    pass

try:
    from Products.CMFQuickInstallerTool.utils import get_packages
except ImportError:
    pass


DirectoryView.registerDirectory("skins", product_globals)
DirectoryView.registerDirectory("skins/CPUtils", product_globals)


def getQIFilteringInformation(self):
    from AccessControl.SecurityManagement import getSecurityManager

    doFiltering = True
    hiddenProducts = ["def"]
    shownProducts = ["def"]
    user = getSecurityManager().getUser()
    portal = getToolByName(self, "portal_url").getPortalObject()
    if user.__module__ == "Products.PluggableAuthService.PropertiedUser":
        doFiltering = False
    if hasattr(portal, "hiddenProducts"):
        hp = list(portal.hiddenProducts)
        hiddenProducts += [p.strip() for p in hp]
    if hasattr(portal, "shownProducts"):
        sp = list(portal.shownProducts)
        shownProducts += [p.strip() for p in sp]
    return doFiltering, hiddenProducts, shownProducts


def listInstallableProducts25(self, skipInstalled=1):
    """List candidate CMF products for
    installation -> list of dicts with keys: (id, hasError, status)
    """
    try:
        from zpi.zope import not_installed

        # print 'Packman support(hotplug) installed'
    except ImportError:

        def not_installed(s):
            return []

    # reset the list of broken products
    self.errors = {}
    pids = self.Control_Panel.Products.objectIds() + not_installed(self)
    pids = [pid for pid in pids if self.isProductInstallable(pid)]

    if skipInstalled:
        installed = [p["id"] for p in self.listInstalledProducts(showHidden=1)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation

    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        p = self._getOb(r, None)
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        if p:
            res.append({"id": r, "status": p.getStatus(), "hasError": p.hasError()})
        else:
            res.append({"id": r, "status": "new", "hasError": 0})
    res.sort(lambda x, y: cmp(x.get("id", None), y.get("id", None)))
    return res


def listInstalledProducts25(self, showHidden=0):
    """Returns a list of products that are installed -> list of
    dicts with keys: (id, hasError, status, isLocked, isHidden,
    installedVersion)
    """
    pids = [
        o.id
        for o in self.objectValues()
        if o.isInstalled() and (o.isVisible() or showHidden)
    ]

    from Products.CPUtils.__init__ import getQIFilteringInformation

    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        p = self._getOb(r, None)
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        res.append(
            {
                "id": r,
                "status": p.getStatus(),
                "hasError": p.hasError(),
                "isLocked": p.isLocked(),
                "isHidden": p.isHidden(),
                "installedVersion": p.getInstalledVersion(),
            }
        )
    res.sort(lambda x, y: cmp(x.get("id", None), y.get("id", None)))
    return res


def listInstallableProducts31(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys: (id, title, hasError, status)
    """
    # reset the list of broken products
    self.errors = {}

    # Get product list from control panel
    pids = self.Control_Panel.Products.objectIds()
    pids = [p for p in pids if self.isProductInstallable(p)]

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()
    profile_pids = [p for p in profile_pids if self.isProductInstallable(p)]
    for p in profile_pids:
        if p.startswith("Products."):
            p = p[9:]
        if p not in pids:
            pids.append(p)

    if skipInstalled:
        installed = [i["id"] for i in self.listInstalledProducts(showHidden=True)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation

    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile["title"]
        if p:
            res.append(
                {
                    "id": r,
                    "title": name,
                    "status": p.getStatus(),
                    "hasError": p.hasError(),
                }
            )
        else:
            res.append({"id": r, "title": name, "status": "new", "hasError": False})
    res.sort(
        lambda x, y: cmp(
            x.get("title", x.get("id", None)), y.get("title", y.get("id", None))
        )
    )
    return res


def listInstalledProducts31(self, showHidden=False):
    """Returns a list of products that are installed -> list of
    dicts with keys: (id, title, hasError, status, isLocked, isHidden,
    installedVersion)
    """
    pids = [
        o.id
        for o in self.objectValues()
        if o.isInstalled() and (o.isVisible() or showHidden)
    ]
    pids = [pid for pid in pids if self.isProductInstallable(pid)]

    from Products.CPUtils.__init__ import getQIFilteringInformation

    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile["title"]

        res.append(
            {
                "id": r,
                "title": name,
                "status": p.getStatus(),
                "hasError": p.hasError(),
                "isLocked": p.isLocked(),
                "isHidden": p.isHidden(),
                "installedVersion": p.getInstalledVersion(),
            }
        )
    res.sort(
        lambda x, y: cmp(
            x.get("title", x.get("id", None)), y.get("title", y.get("id", None))
        )
    )
    return res


def listInstallableProducts40(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys: (id, title, hasError, status)
    """
    # reset the list of broken products
    try:
        self.errors = {}
    except AttributeError:
        pass

    # Returns full names with Products. prefix for all packages / products
    packages = get_packages()

    pids = []
    for p in packages:
        if not self.isProductInstallable(p):
            continue
        if p.startswith("Products."):
            p = p[9:]
        pids.append(p)

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()

    for p in profile_pids:
        if p in pids or p in packages:
            continue
        if not self.isProductInstallable(p):
            continue
        pids.append(p)

    if skipInstalled:
        installed = [p["id"] for p in self.listInstalledProducts(showHidden=True)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation

    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile["title"]
        if p:
            res.append(
                {
                    "id": r,
                    "title": name,
                    "status": p.getStatus(),
                    "hasError": p.hasError(),
                }
            )
        else:
            res.append({"id": r, "title": name, "status": "new", "hasError": False})
    res.sort(
        lambda x, y: cmp(
            x.get("title", x.get("id", None)), y.get("title", y.get("id", None))
        )
    )
    return res


def listInstallableProducts434(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys: (id, title, hasError, status)
    """
    # reset the list of broken products
    if getattr(self, "_v_errors", True):
        self._v_errors = {}

    # Returns full names with Products. prefix for all packages / products
    packages = get_packages()

    pids = []
    for p in packages:
        if not self.isProductInstallable(p):
            continue
        if p.startswith("Products."):
            p = p[9:]
        pids.append(p)

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()

    for p in profile_pids:
        if p in pids or p in packages:
            continue
        if not self.isProductInstallable(p):
            continue
        pids.append(p)

    if skipInstalled:
        installed = [p["id"] for p in self.listInstalledProducts(showHidden=True)]
        pids = [r for r in pids if r not in installed]

    from Products.CPUtils.__init__ import getQIFilteringInformation

    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    res = []
    for r in pids:
        if doFiltering and r in hiddenProducts and r not in shownProducts:
            continue
        p = self._getOb(r, None)
        name = r
        profile = self.getInstallProfile(r)
        if profile:
            name = profile["title"]
        if p:
            res.append(
                {
                    "id": r,
                    "title": name,
                    "status": p.getStatus(),
                    "hasError": p.hasError(),
                }
            )
        else:
            res.append({"id": r, "title": name, "status": "new", "hasError": False})
    res.sort(
        lambda x, y: cmp(
            x.get("title", x.get("id", None)), y.get("title", y.get("id", None))
        )
    )
    return res


def listInstallableProducts437(self, skipInstalled=True):
    """List candidate CMF products for installation -> list of dicts
       with keys:(id,title,hasError,status)
    """
    self._init_errors(reset=True)

    # Returns full names with Products. prefix for all packages / products
    packages = get_packages()

    pids = []
    for pkg in packages:
        if not self.isProductInstallable(pkg):
            continue
        if pkg.startswith("Products."):
            pkg = pkg[9:]
        pids.append(pkg)

    # Get product list from the extension profiles
    profile_pids = self.listInstallableProfiles()

    for pp in profile_pids:
        if pp in pids or pp in packages:
            continue
        if not self.isProductInstallable(pp):
            continue
        pids.append(pp)

    from Products.CPUtils.__init__ import getQIFilteringInformation

    (doFiltering, hiddenProducts, shownProducts) = getQIFilteringInformation(self)

    if skipInstalled:
        installed = [p["id"] for p in self.listInstalledProducts(showHidden=True)]
        pids = [r for r in pids if r not in installed]

    res = []
    for pid in pids:
        if doFiltering and pid in hiddenProducts and pid not in shownProducts:
            continue
        installed_product = self._getOb(pid, None)
        name = pid
        profile = self.getInstallProfile(pid)
        if profile:
            name = profile["title"]
        record = {"id": pid, "title": name}
        if installed_product:
            record["status"] = installed_product.getStatus()
            record["hasError"] = installed_product.hasError()
        else:
            record["status"] = "new"
            record["hasError"] = False
        res.append(record)
    res.sort(
        lambda x, y: cmp(
            x.get("title", x.get("id", None)), y.get("title", y.get("id", None))
        )
    )
    return res


def CallMaxSizeValidator(self, value, *args, **kwargs):
    instance = kwargs.get("instance", None)
    field = kwargs.get("field", None)
    type_doc = instance.getPortalTypeName().replace(" ", "").lower() + "_maxsize"
    # get max size
    if "maxsize" in kwargs:
        maxsize = kwargs.get("maxsize")
    elif hasattr(aq_base(instance), "getMaxSizeFor"):
        maxsize = instance.getMaxSizeFor(field.getName())
    elif hasattr(field, "maxsize"):
        maxsize = field.maxsize
    elif hasattr(instance, type_doc):
        maxsize = getattr(instance, type_doc)
    else:
        # set to given default value (default defaults to 0)
        maxsize = self.maxsize

    if not maxsize:
        return True

    # calculate size
    elif (
        isinstance(value, FileUpload)
        or isinstance(value, file)
        or hasattr(aq_base(value), "tell")
    ):
        value.seek(0, 2)  # eof
        size = value.tell()
        value.seek(0)
    else:
        try:
            size = len(value)
        except TypeError:
            size = 0
    size = float(size)
    sizeMB = size / (1024 * 1024)

    if sizeMB > maxsize:
        msg = _(
            "Validation failed($name: Uploaded data is too large: ${size}MB (max ${max}MB)",
            mapping={
                "name": safe_unicode(self.name),
                "size": safe_unicode("%.3f" % sizeMB),
                "max": safe_unicode("%.3f" % maxsize),
            },
        )
        return recursiveTranslate(msg, **kwargs)
    else:
        return True


def initialize(context):
    logger.info("ADDING MONKEY PATCHS !")
    if not PLONE_VERSION:
        logger.error("CMFPlone version NOT FOUND: MONKEY PATCH NOT APPLIED")
        return
    elif PLONE_VERSION.startswith("2.5"):
        QuickInstallerTool.listInstallableProducts = listInstallableProducts25
        QuickInstallerTool.listInstalledProducts = listInstalledProducts25
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
    elif PLONE_VERSION.startswith("3."):
        QuickInstallerTool.listInstallableProducts = listInstallableProducts31
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
    elif PLONE_VERSION < "4.3.4":
        QuickInstallerTool.listInstallableProducts = listInstallableProducts40
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        try:
            # Patching tinymce to load as html in Ploneboard
            from Products.TinyMCE.utility import TinyMCE

            old_getContentType = TinyMCE.getContentType

            def getContentType(self, object=None, field=None, fieldname=None):
                if (
                    object is not None
                    and fieldname == "text"
                    and object.meta_type
                    in ("PloneboardForum", "PloneboardConversation")
                ):
                    return "text/html"
                return old_getContentType(
                    self, object=object, field=field, fieldname=fieldname
                )

            # TinyMCE.getContentType = getContentType
            logger.info(
                "TinyMCE getContentType MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION
            )
        except BaseException:
            pass
    elif PLONE_VERSION >= "4.3.4":
        QuickInstallerTool.listInstallableProducts = listInstallableProducts434
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
    elif PLONE_VERSION >= "4.3.7":
        QuickInstallerTool.listInstallableProducts = listInstallableProducts437
        QuickInstallerTool.listInstalledProducts = listInstalledProducts31
        logger.info("QuickInstallerTool MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)
        MaxSizeValidator.__call__ = CallMaxSizeValidator
        logger.info("MaxSizeValidator MONKEY PATCHED FOR PLONE %s!" % PLONE_VERSION)



# -*- coding: utf-8 -*-
from Products.CPUtils.config import PROJECTNAME
from Products.CPUtils.tests.CPUtilsTestCase import CPUtilsTestCase
from Products.PloneTestCase import PloneTestCase
from Testing import ZopeTestCase

import os
import sys


if __name__ == "__main__":
    execfile(os.path.join(sys.path[0], "framework.py"))

ZopeTestCase.installProduct(PROJECTNAME)

PRODUCTS = list()
PRODUCTS.append(PROJECTNAME)
productName = "PloneGazette"
PRODUCTS.append(productName)

testcase = PloneTestCase.PloneTestCase
# Extension profile doesn't work because appinstall of ts appears to be not run
# PloneTestCase.setupPloneSite(products=PRODUCTS, extension_profiles=EXTENSION_PROFILES)
PloneTestCase.setupPloneSite(products=PRODUCTS)


class testNewsLetter(CPUtilsTestCase):
    """Test-cases for class(es) ."""
    def afterSetUp(self):
        """
        """
        CPUtilsTestCase.afterSetup(self)
        self.loginAsPortalOwner()

        """from Products.Five import zcml
        from Products.Five import fiveconfigure
        fiveconfigure.debug_mode=True
        import Products.PloneGazette
        zcml.load_config('configure.zcml', Products.PloneGazette)
        fiveconfigure.debug_mode=False
        self.addProduct(name)"""
        self.portal.portal_quickinstaller

    # Manually created methods

    def test_list_newsletter_users(self):
        print(
            "problème pour créer une newsletter et ses abonnés: invokeFactory() ne fonctionne pas "
        )
        print("(probablement car plonegazette ne s'installe pas correctement)")
        """
        self.portal.invokeFactory('NewsletterTheme', id)
        nlf = self.portal.newsletter

        nlf.invokeFactory('NewsletterBTree', id='subscribers')
        nlf.subscriber_folder_id = 'subscribers'

        nlf.createSubscriberObject('sub1')
        nlf.createSubscriberObject('sub2')
        nlf.createSubscriberObject('sub3')
        subList = nlf.getSubscribers()
        print(subList)"""
        self.failUnless(True)


def test_suite():
    from unittest import makeSuite
    from unittest import TestSuite

    suite = TestSuite()
    suite.addTest(makeSuite(testNewsLetter))
    return suite


if __name__ == "__main__":
    import framework
    framework()



#!/usr/bin/python
from utils import error
from utils import packdb
from utils import read_zopeconffile
from utils import verbose

import os
import shutil
import sys


buildout_inst_type = None  # True for buildout, False for manual instance


def main():
    global buildout_inst_type

    tmp = instdir
    if tmp.endswith("/"):
        tmp = tmp[0:-1]
    verbose("Working on instance %s" % tmp)

    if os.path.exists(os.path.join(tmp, "parts")):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(tmp, "etc")):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(tmp) or True:
        error("! Invalid instance path '%s' or instance type not detected" % tmp)
        sys.exit(1)

    if buildout_inst_type:
        zodbfilename = instdir + "/parts/instance/etc/zope.conf"
    else:
        zodbfilename = instdir + "/etc/zope.conf"

    lines = []
    read_zopeconffile(zodbfilename, lines)

    port = ""
    httpflag = False
    dbs = []
    for line in lines:
        # verbose("=>'%s'"%line)
        if line.startswith("<http-server>"):
            httpflag = True
            continue
        if httpflag and line.startswith("address"):
            port = line.split()[1]
            httpflag = False
            continue
        if line.startswith("<zodb_db"):
            dbname = line.split()[1]
            dbname = dbname.rstrip(">")
            if dbname != "temporary":
                dbs.append(dbname)
    verbose("\tport='%s', dbs='%s'" % (port, ";".join(dbs)))

    if not port:
        error("! the port was not found in the config file '%s'" % zodbfilename)

    packdb(port, dbs, days, "cputils_pack_db", "CPUtils.utils", "pack_db")

    # we delete every .old created during packing in the /var directory of the zope instance
    if buildout_inst_type:
        dir_path = instdir + "/var/filestorage/"
    else:
        dir_path = instdir + "/var/"

    for file in shutil.os.listdir(dir_path):
        if file.endswith(".fs.old"):
            # we do no more delete fs.old to safely backup this file as zodb file
            # shutil.os.unlink(dir_path + file)
            # verbose("\t%s deleted" % (dir_path + file))
            pass
        elif file.endswith(".fs.pack"):
            shutil.os.unlink(dir_path + file)
            error("! .pack file found : pack not correctly ended")
            verbose("\t%s deleted" % (dir_path + file))


# ------------------------------------------------------------------------------

try:
    arg = sys.argv[1]
    if arg.startswith("#"):
        sys.exit(0)
    instdir, days, user, pwd = arg.split(";")
    verbose("Start of packing %s, days=%s" % (instdir, days))
except IndexError:
    error("No parameter found")
    sys.exit(1)
except ValueError:
    error("No enough parameters")
    sys.exit(1)

if __name__ == "__main__":
    main()
    verbose("End of packing %s" % (instdir))



global dic_hpList
dic_hpList = {
    "acptheme.cpskin3": [
        "ATVocabularyManager",
        "CacheSetup",
        "CMFSquidTool",
        "Clouseau",
        "CPUtils",
        "ImageEditor",
        "LinguaPlone",
        "Marshall",
        "NuPlone",
        "Ploneboard",
        "RICDescriptive",
        "SimpleAttachment",
        "SmartColorWidget",
        "ZopeRepository",
        "collective.js.jquery",
        "collective.js.jqueryui",
        "plone.app.iterate",
        "plone.app.openid",
        "plone.app.z3cform",
    ],
    "urban": ["produit 1", "produit 3", "produit 4"],
    "PloneMeeting:": [],
    "TeleServices": [],
}



# -*- coding: utf-8 -*-
from Products.CPUtils.tests.CPUtilsTestCase import CPUtilsTestCase

import os
import sys


if __name__ == "__main__":
    execfile(os.path.join(sys.path[0], "framework.py"))


class testMethods(CPUtilsTestCase):
    """Test-cases for class(es) ."""

    def afterSetUp(self):
        """
        """
        CPUtilsTestCase.afterSetup(self)

        self.loginAsPortalOwner()

    # Manually created methods

    def test_audit_catalog(self):
        result = self.portal.cputils_audit_catalog()
        ok = result.startswith(
            "<h1>RESULTATS DE LA RECHERCHE</h1> <p>Recherche dans : /</p> <p>Nombre d'elements trouves :"
        )
        ok = ok and result.find("portal_type : getObjSize : url") > -1
        self.failUnless(ok)

    def test_change_authentication_plugins_param_doChange(self):
        result = self.app.cputils_change_authentication_plugins(self.app, "", "")
        ok = result.startswith(
            "The following changes are not applied: you must run the script with the parameter '...?dochange=1'"
        )
        result = self.app.cputils_change_authentication_plugins(self.app, 1, "")
        ok = ok and result.startswith(
            "The following changes are not applied: you must run the script with the parameter '...?dochange=1'"
        )
        self.failUnless(ok)

    def test_change_authentication_plugins_param_activate(self):
        result = self.app.cputils_change_authentication_plugins(self.app, "", 1)
        ok = result.startswith("Document '/authentication_plugins_sites' added")
        ok = ok and result.find("Desactivate plugins source_users") > -1
        result = self.app.cputils_change_authentication_plugins(self.app, 1, 1)
        ok = ok and result.find("site found plone")
        ok = ok and result.find("Activate plugins source_users") > -1
        self.failUnless(ok)

    def test_change_user_properties_no_param(self):
        result = self.portal.cputils_change_user_properties(self.portal, "", "")
        ok = (
            result.find("USER:'member'") > -1
            and result.find("USER:'admin'") > -1
            and result.find("USER:'anon'") > -1
        )
        self.failUnless(ok)

    def test_change_user_properties_param_visibleIDs(self):
        result = self.portal.cputils_change_user_properties(
            self.portal, "visible_ids:True", ""
        )
        ok = (
            result.find("USER:'member'") > -1
            and result.find("USER:'admin'") > -1
            and result.find("USER:'anon'") > -1
        )
        ok = ok and result.find(" all properties: visible_ids='False'")
        result = self.portal.cputils_change_user_properties(
            self.portal, "visible_ids:True", 1
        )
        ok = (
            result.find("USER:'member'") > -1
            and result.find("USER:'admin'") > -1
            and result.find("USER:'anon'") > -1
        )
        ok = ok and result.find(
            "old properties: visible_ids='False',<br/>->  new properties: visible_ids='True',"
        )
        self.failUnless(ok)

    def test_cpdb(self):
        print("push 'c' and then 'return' to pass the test")
        # self.portal.cputils_cpdb()
        "cputils_cpdb test passed"

    def old_test_desactivate_base2dom(self):
        result = self.app.cputils_desactivate_base2dom()
        ok = result.find("Disabled ++resource++base2-dom-fp.js for /plone") > -1
        self.failUnless(ok)

    def old_test_list_portlets(self):
        result = self.portal.cputils_list_portlets()
        ok = result.startswith(
            "left: {u'login': <Assignment at login>, u'navigation': <Assignment at navigation>}"
        )
        ok = ok and result.endswith(
            "right: {u'news': <Assignment at news>, u'review': <Assignment at review>, u'events': <Assignment at events>, u'calendar': <Assignment at calendar>}"
        )
        self.failUnless(ok)

    def old_test_list_users_param_sort(self):
        result = self.portal.cputils_list_users(self.portal, "csv", "users")
        ok = result.endswith(
            "<br />UserId,GroupId<br />admin,AuthenticatedUsers<br />member,AuthenticatedUsers<br />anon,AuthenticatedUsers<br />test_user_1_,AuthenticatedUsers"
        )
        # result = self.portal.cputils_list_users('csv','groups')
        self.failUnless(ok)

    def old_test_store_user_properties(self):
        self.portal.cputils_store_user_properties()
        ok = str(self.portal.users_properties).find("False	Kupu") > -1
        self.portal.cputils_change_user_properties(self.portal, "visible_ids:True", 1)
        self.portal.cputils_store_user_properties()
        ok = ok and str(self.portal.users_properties).find("True	Kupu") > -1
        self.failUnless(ok)


def test_suite():
    from unittest import makeSuite
    from unittest import TestSuite

    suite = TestSuite()
    suite.addTest(makeSuite(testMethods))
    return suite


if __name__ == "__main__":
    import framework
    framework()



#!/usr/bin/python
#
import datetime
import os


def verbose(*messages):
    print ">>", " ".join(messages)


def error(*messages):
    #    print >>sys.stderr, '!!', (' '.join(messages))
    print "!!", (" ".join(messages))


MAIL = "/usr/sbin/sendmail"
logfilenames = (
    "/var/log/cron_scripts.log",
    "/var/log/checkInstances.log",
    "/var/log/checkPoskey.log",
)
hostname = "mydomain.be"
today = datetime.date.today().strftime("%Y-%m-%d")
yesterday = (datetime.date.today() - datetime.timedelta(1)).strftime("%Y-%m-%d")
header = (
    "To: server@communesplone.be",
    "From: root@%s" % hostname,
)


def read_log_file(filename, lines):
    """
        Read the log file
    """
    try:
        logfile = open(filename, "r")
        verbose("Reading file '%s'" % filename)
    except IOError:
        error("Cannot open %s file" % filename)
        return
    flag_tokeep = False
    for line in logfile.readlines():
        line = line.strip("\n\t ")
        # print line
        if line.startswith("##  %s, 23" % yesterday):
            flag_tokeep = True
        if line.startswith("##  %s," % today):
            flag_tokeep = True
        if flag_tokeep:
            lines.append(line)


verbose("Searching in log between %s and %s" % (yesterday, today))

for logfilename in logfilenames:
    bodylines = [""]
    read_log_file(logfilename, bodylines)
    # when logrotate is run, the logfile is cleared and
    # renamed in .1 (with 'delaycompress' option for logrotate)
    # we open the .1 file to get those lines
    if len(bodylines) < 10 and os.path.exists(logfilename + ".1"):
        rotatedlines = [""]
        read_log_file(logfilename + ".1", rotatedlines)
        bodylines[0:0] = rotatedlines

    # print '\n'.join(bodylines)
    # sys.exit(0)

    subject = "\nSubject: %s %s from %s" % (
        today,
        os.path.basename(logfilename),
        hostname,
    )

    # open a pipe to the mail program and
    # write the data to the pipe
    p = os.popen("%s -t" % MAIL, "w")
    #    p = open("%s"%os.path.basename(logfilename), 'w')  #testing
    p.write("\n".join(header))
    p.write(subject)
    p.write("\n".join(bodylines))
    exitcode = p.close()
    if exitcode:
        print "Exit code: %s" % exitcode



# -*- coding: utf-8 -*-
from Globals import package_home
from Products.CMFPlone.tests import PloneTestCase
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite
from zope.testing import doctest

import glob
import os
import sys
import unittest


if __name__ == "__main__":
    execfile(os.path.join(sys.path[0], "framework.py"))

PACKAGE = "Products.CPUtils.tests"

REQUIRE_TESTBROWSER = ["tst_CPUtils.txt"]

OPTIONFLAGS = (
    doctest.REPORT_ONLY_FIRST_FAILURE | doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
)


def list_doctests():
    home = package_home(globals())
    print home
    return [filename for filename in glob.glob(os.path.sep.join([home, "*.txt"]))]


def list_nontestbrowser_tests():
    return [
        filename
        for filename in list_doctests()
        if os.path.basename(filename) not in REQUIRE_TESTBROWSER
    ]


def setUp(self):
    uf = self.portal.acl_users
    uf.userFolderAddUser("member", "member", ["Member", ], [])
    uf.userFolderAddUser("admin", "admin", ["Manager", "Member", ], [])
    uf.userFolderAddUser("anon", "anon", ["Anonymous", ], [])
    # we have to be logged as a Manager to run methods from exportimport
    self.login("admin")
    self.logout()
    return None


def test_suite():
    filenames = list_doctests()
    suites = [
        Suite(
            os.path.basename(filename),
            optionflags=OPTIONFLAGS,
            package=PACKAGE,
            setUp=setUp,
            test_class=PloneTestCase.FunctionalTestCase,
        )
        for filename in filenames
    ]

    # BBB: Fix for http://zope.org/Collectors/Zope/2178
    from Products.PloneTestCase import layer
    from Products.PloneTestCase import setup

    if setup.USELAYER:
        for s in suites:
            if not hasattr(s, "layer"):
                s.layer = layer.PloneSite

    return unittest.TestSuite(suites)


if __name__ == "__main__":
    import framework
    framework()



#!/usr/bin/python
#
"""
    This script reads awstats conf files and updates stats with all apache log files (archived too).
"""

import datetime
import os
import shutil
import sys


def verbose(*messages):
    print ">>", " ".join(messages)


def error(*messages):
    #    print >>sys.stderr, '!!', (' '.join(messages))
    print "!!", (" ".join(messages))


def debug(*messages):
    if not TRACE:
        return
    print "TRACE:", " ".join(messages)


# ------------------------------------------------------------------------------

# CONF_DIR = '/etc/awstats'
CONF_DIR = "/home/srv/temp/awstats"
# APACHE_DIR = '/var/log/apache2'
APACHE_DIR = "/home/srv/temp/apache2_logs"
INCLUDE_ARCHIVED = False
AWSTATS_CMD = "/usr/lib/cgi-bin/awstats.pl"
TRACE = False
TEMP_DIR = "/tmp/"
gunzip = "gunzip"

# ------------------------------------------------------------------------------


def getFiles(dirpath, extensions, namestart, sort=False):
    """ Read the dir and return some files """
    files = []
    for filename in os.listdir(dirpath):
        debug("filename='%s'" % filename)
        if filename in (".", ".."):
            continue
        extension = os.path.splitext(filename)[1].strip(".")
        debug("extension='%s'" % extension)
        if not extension:
            continue
        if extensions and extension not in extensions:
            continue
        if namestart and not filename.startswith(namestart):
            continue
        debug("Kept '%s'" % filename)
        filepath = os.path.join(dirpath, filename)
        files.append(filepath)
    if sort:
        files.sort(compare_file_modiftime)
    return files


# -------------------------------------------------


def readFile(filename, lines):
    """ read the filename and return lines """
    try:
        zfile = open(filename, "r")
    except IOError:
        error("! Cannot open %s file" % filename)
        return
    #    lines = zfile.readlines()
    for line in zfile.readlines():
        line = line.strip("\n\t ")
        lines.append(line)
    zfile.close()


def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    os.system(cmd + " >_cmd_pv.out 2>_cmd_pv.err")
    stdout = stderr = []
    try:
        if os.path.exists("_cmd_pv.out"):
            ofile = open("_cmd_pv.out", "r")
            stdout = ofile.readlines()
            ofile.close()
            os.remove("_cmd_pv.out")
        else:
            error("File %s does not exist" % "_cmd_pv.out")
    except IOError:
        error("Cannot open %s file" % "_cmd_pv.out")
    try:
        if os.path.exists("_cmd_pv.err"):
            ifile = open("_cmd_pv.err", "r")
            stderr = ifile.readlines()
            ifile.close()
            os.remove("_cmd_pv.err")
        else:
            error("File %s does not exist" % "_cmd_pv.err")
    except IOError:
        error("Cannot open %s file" % "_cmd_pv.err")
    return (stdout, stderr)


# ------------------------------------------------------------------------------


def compare_file_modiftime(file1, file2):
    """ compare file in function of modification time """
    date1 = datetime.datetime.fromtimestamp(os.stat(file1).st_mtime)
    date2 = datetime.datetime.fromtimestamp(os.stat(file2).st_mtime)
    return cmp(date1, date2)


# ------------------------------------------------------------------------------


def main():
    verbose("Reading conf files")
    for conffilepath in getFiles(CONF_DIR, ("conf",), "", sort=True):
        conffile = os.path.basename(conffilepath)
        verbose("Reading %s" % conffile)
        lines = []
        readFile(conffilepath, lines)
        for line in lines:
            line = line.strip("\n\t ")
            #            debug("line '%s'"%line)
            if line.startswith("LogFile"):
                line = line[7:].strip('=" ')
                # to avoid line like LogFile="/usr/share/doc/awstats/examples/logresolvemerge.pl /srv/apache/apache2_logs/*_access.log |"
                if line.find("|") >= 0:
                    break
                logfilename = os.path.basename(line)
                verbose("\tFound apache log name '%s'" % logfilename)
                logfiles = getFiles(APACHE_DIR, [], logfilename, sort=True)
                if not logfiles:
                    error("No log files found in '%s'" % APACHE_DIR)
                # we remove begin and end: awstats. xxx .conf
                configname = conffile[8:-5]
                verbose("\tconfig name = '%s'" % configname)
                for logfilepath in logfiles:
                    logfile = os.path.basename(logfilepath)
                    if not (
                        logfile.endswith(".log")
                        or logfile.endswith(".log.1")
                        or logfile.endswith(".gz")
                    ):
                        error("Logfile found but not recognized '%s'" % logfile)
                        continue
                    verbose("\tLogfile found '%s'" % logfile)
                    if logfile.endswith(".gz"):
                        # we need to decompress the file
                        destination = os.path.join(TEMP_DIR, logfile)
                        try:
                            shutil.copyfile(logfilepath, destination)
                            verbose(
                                "\t'%s' copied to '%s'" % (logfilepath, destination)
                            )
                        except Exception, errmsg:
                            error(
                                "'%s' NOT COPIED to '%s'" % (logfilepath, destination)
                            )
                            error(str(errmsg))
                        command = "%s -f %s" % (gunzip, destination)
                        (cmd_out, cmd_err) = runCommand(command)
                        if cmd_err:
                            error(
                                "error running command %s : %s"
                                % (command, "".join(cmd_err))
                            )
                        logfilepath = destination[:-3]
                    command = "%s -config=%s -Logfile=%s -update" % (
                        AWSTATS_CMD,
                        configname,
                        logfilepath,
                    )
                    verbose("\t>> Running '%s'" % command)
                    (cmd_out, cmd_err) = runCommand(command)
                    if cmd_err:
                        error(
                            "error running command %s : %s"
                            % (command, "".join(cmd_err))
                        )
                    if cmd_out:
                        verbose("\t>>OUTPUT: %s" % ("".join(cmd_out)))
                    if logfile.endswith(".gz"):
                        os.remove(logfilepath)
                        verbose("\t'%s' deleted" % logfilepath)
                break
    verbose("End of script")


# ------------------------------------------------------------------------------

try:
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option(
        "-a",
        "--archived",
        dest="archived",
        default=False,
        help="Include archived logs or not. Default=False",
    )
    (options, args) = parser.parse_args()
    INCLUDE_ARCHIVED = options.archived
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# André NUYENS <andre.nuyens@uvcw.be>, UVCW
#

from datetime import datetime

import os
import sys
import urllib


TRACE = False


def verbose(*messages):
    print ">>", " ".join(messages)


def error(*messages):
    print "!!", (" ".join(messages))


def trace(*messages):
    if not TRACE:
        return
    print "TRACE:", " ".join(messages)


# ------------------------------------------------------------------------------


def read_zopectlfile(zopectlfilename):
    """ read the zopectl file to find the zope path """
    try:
        zfile = open(zopectlfilename, "r")
    except IOError:
        error("! Cannot open %s file" % zopectlfilename)
        return
    zopepath = ""
    pythonfile = ""
    for line in zfile.readlines():
        line = line.strip("\n\t ")
        if line.startswith("ZOPE_HOME"):
            zopepath = line.split("=")[1]
            zopepath = zopepath.strip("\"' ")
        elif line.startswith("PYTHON="):
            pythonfile = line.split("=")[1]
            pythonfile = pythonfile.strip("\"' ")
    return (zopepath, pythonfile)


# ------------------------------------------------------------------------------


def read_zopeconffile(zodbfilename, lines):
    """ read the zope conf filename and include subfile """
    try:
        zfile = open(zodbfilename, "r")
    except IOError:
        error("! Cannot open %s file" % zodbfilename)
        return
    for line in zfile.readlines():
        line = line.strip("\n\t ")
        if line.startswith("%include"):
            otherfilename = line.split()[1]
            read_zopeconffile(otherfilename, lines)
            continue
        lines.append(line)
    zfile.close()


# ------------------------------------------------------------------------------


def treat_zopeconflines(zodbfilename, fspath):
    """
        read zope configuration lines to get informations
    """
    lines = []
    read_zopeconffile(zodbfilename, lines)

    httpflag = False
    mp_name = mp_path = mp_fs = mp_fspath = None
    for line in lines:
        line = line.strip()
        if line.startswith("<http-server>"):
            httpflag = True
            continue
        if line.startswith("<zodb_db "):
            # <zodb_db main>
            if mp_name:
                error(
                    "\tnext db found while end tag not found: previous dbname '%s', current line '%s'"
                    % (mp_name, line)
                )
            mp_name = line.split()[1]
            mp_name = mp_name.strip("> ")
            if mp_name == "temporary":
                mp_name = None
            continue
        if mp_name and line.startswith("</zodb_db>"):
            mp_name = None
            continue
        if mp_name and line.startswith("path "):
            # path $INSTANCE/var/Data.fs
            mp_fs = os.path.basename(line.split()[1])
            if not mp_fs.endswith(".fs"):
                error("Error getting fs name in '%s'" % line)
            mp_fspath = os.path.join(fspath, mp_fs)
            if os.path.exists(mp_fspath):
                int(os.path.getsize(mp_fspath) / 1048576)
            else:
                error("Db file '%s' doesn't exist" % mp_fspath)
            continue
        if mp_name and line.startswith("mount-point "):
            # mount-point /
            mp_path = line.split()[1]
            if not mp_path.startswith("/"):
                error("Error getting path name in '%s'" % line)
            continue
        if httpflag and line.startswith("address"):
            port = line.split()[1]
    return port


class MyUrlOpener(urllib.FancyURLopener):
    """ redefinition of this class to give the user and password"""

    def prompt_user_passwd(self, host, realm):
        return (user, pwd)

    def __init__(self, *args):
        self.version = "Zope Packer"
        urllib.FancyURLopener.__init__(self, *args)


# ------------------------------------------------------------------------------


def packdb(port, db, days, method, module, function, user_, pwd_):
    global user, pwd
    user = user_
    pwd = pwd_
    host = "http://localhost:%s" % port
    urllib._urlopener = MyUrlOpener()
    start = datetime.now()
    verbose("\tPacking db '%s' for instance %s (%s days)" % (db, host, days))
    url_spd = "%s/Control_Panel/Database/%s/%s?days:float=%s" % (host, db, method, days)
    # verbose("url='%s'"%url_spd)
    try:
        ret_html = urllib.urlopen(url_spd).read()
        if (
            "the requested resource does not exist" in ret_html
            or "error was encountered while publishing this resource" in ret_html
        ):
            verbose("\texternal method %s not exist : we will create it" % method)
            url_em = (
                "%s/manage_addProduct/ExternalMethod/manage_addExternalMethod?id=%s&module=%s&function=%s&title="
                % (host, method, module, function)
            )
            try:
                ret_html = urllib.urlopen(url_em).read()
                if (
                    "the requested resource does not exist" in ret_html
                    or "error was encountered while publishing this resource"
                    in ret_html
                ):
                    error("! Cannot create external method in zope")
                elif "The specified module," in ret_html:
                    error(
                        "! The specified module %s is not present in the instance"
                        % module
                    )
                else:
                    try:
                        ret_html = urllib.urlopen(url_spd).read()
                        #                            if "/Control_Panel/Database/%s"%db not in ret_html:
                        #                                error("Problem during compression of %s"%db)
                        #                                log.debug(ret_html)
                        verbose("\t%s" % ret_html)
                    except IOError, msg:
                        error("! Cannot open URL %s, aborting : %s" % (url_spd, msg))
            except Exception, msg:
                error("! Cannot open URL %s, aborting : %s" % (url_em, msg))
        else:
            verbose("\t%s" % ret_html)
    except IOError, msg:
        error("! Cannot open URL %s, aborting : %s" % (url_spd, msg))
    verbose("\t\t-> elapsed time %s" % (datetime.now() - start))


# ------------------------------------------------------------------------------


def CreateAndCallExternalMethod(
    port, user_, pwd_, ext_method, ext_filename, function, param=""
):
    # creating and calling external method in zope
    global instdir, tempdir, buildout_inst_type, user, pwd
    host = "http://localhost:%s" % port
    user = user_
    pwd = pwd_
    urllib._urlopener = MyUrlOpener()
    url_pv = "%s/%s%s" % (host, ext_method, param)
    current_url = url_pv
    try:
        # verbose("Running '%s'"%current_url)
        ret_html = urllib.urlopen(current_url).read()
        if "the requested resource does not exist" in ret_html:
            verbose("external method %s not exist : we will create it" % ext_method)
            (module, extension) = os.path.splitext(ext_filename)
            module = "CPUtils." + module
            current_url = (
                "%s/manage_addProduct/ExternalMethod/manage_addExternalMethod?id=%s&module=%s&function=%s&title="
                % (host, ext_method, module, function)
            )
            # verbose("Running now '%s'"%current_url)
            ret_html = urllib.urlopen(current_url).read()
            if "the requested resource does not exist" in ret_html or (
                "The specified module" in ret_html and "couldn't be found" in ret_html
            ):
                error("Cannot create external method in zope : '%s'" % ret_html)
                sys.exit(1)
            else:
                current_url = "%s/%s/valid_roles" % (host, ext_method)
                # verbose("Running now '%s'"%current_url)
                ret_html = urllib.urlopen(current_url).read()
                if not ret_html[0] == "(":
                    error("error with valid_roles return: '%s'" % ret_html)
                    sys.exit(1)
                valid_roles = list(eval(ret_html))
                managerindex = valid_roles.index("Manager")
                current_url = "%s/%s/permission_settings" % (host, ext_method)
                # verbose("Running now '%s'"%current_url)
                ret_html = urllib.urlopen(current_url).read()
                if not ret_html[0] == "[":
                    error("error with permission_settings return: '%s'" % ret_html)
                    sys.exit(1)
                permission_settings = eval(ret_html)
                params = {}
                count = 0
                for perm in permission_settings:
                    if perm["name"] in ("Access contents information", "View"):
                        params["p%dr%d" % (count, managerindex)] = "on"
                    else:
                        params["a%d" % count] = "on"
                    count += 1
                current_url = "%s/%s/manage_changePermissions" % (host, ext_method)
                # verbose("Running now '%s'"%current_url)
                # params example                       params = {  'permission_to_manage':'View',
                #                                    'roles':['Manager'], }
                data = urllib.urlencode(params)
                ret_html = urllib.urlopen(current_url, data).read()
                if "Your changes have been saved" not in ret_html:
                    error(
                        "Error changing permissions with URL '%s', data '%s'"
                        % (current_url, str(data))
                    )
                    sys.exit(1)
                current_url = url_pv
                # verbose("Running again '%s'"%current_url)
                ret_html = urllib.urlopen(current_url).read()
        # verbose("callAndCreateExternalMethod : \n%s"%ret_html)
        if ret_html:
            verbose("%s" % ret_html)
    except Exception, msg:
        error("Cannot open URL %s, aborting: '%s'" % (current_url, msg))
        sys.exit(1)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Script to create and update a repository on zope instances
# Stéphan Geulette <stephan.geulette@uvcw.be>, UVCW
# From original script "recover.py" ()
#
from Products.CPUtils.scripts.utils import CreateAndCallExternalMethod
from Products.CPUtils.scripts.utils import trace
from Products.CPUtils.scripts.utils import treat_zopeconflines
from utils import datetime
from utils import error
from utils import verbose

import os
import socket
import sys


buildout_inst_type = None  # True for buildout, False for manual instance
tempdir = ""
now = datetime(1973, 02, 12).now()
pfolders = {}
temp_added = False


def main():
    global instdir, tempdir, buildout_inst_type, user, pwd

    instdir = instdir.rstrip("/")
    verbose(
        "Working on instance %s" % instdir
    )  # must be commented when CPUtils updated in all instances

    # Finding the instance type (buildout or manual)
    if os.path.exists(os.path.join(instdir, "parts")):
        buildout_inst_type = True
        # verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(instdir, "etc")):
        buildout_inst_type = False
        verbose("%s, manual installation !" % instdir)
    elif not os.path.exists(instdir) or True:
        error("%s, Invalid instance path or instance type not detected" % instdir)
        sys.exit(1)

    if buildout_inst_type:
        zodbfilename = os.path.join(instdir, "parts/instance/etc/zope.conf")
        fspath = os.path.join(instdir, "var/filestorage/")
    else:
        zodbfilename = os.path.join(instdir, "etc/zope.conf")
        fspath = os.path.join(instdir, "var/")

    instance = os.path.basename(instdir)
    if not tempdir:
        tempdir = os.path.join(instdir, "temp")
    hostname = socket.gethostname()

    port = treat_zopeconflines(zodbfilename, fspath)

    trace("host='%s', inst='%s'" % (hostname, instance))

    param = "?instdir=%s" % instdir
    if instance.find("test") < 0:
        param += "&isProductInstance=1"
    CreateAndCallExternalMethod(
        port, user, pwd, "cputils_checkInstance", "utils.py", "checkInstance", param
    )


try:
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option(
        "-i",
        "--infos",
        dest="infos",
        default=None,
        help="infos about instance formatted like: "
        '"instance_path;transactions_days_number;admin_user;admin_password"',
    )
    (options, args) = parser.parse_args()
    if options.infos.startswith("#"):
        sys.exit(0)
    instdir, days, user, pwd = options.infos.split(";")
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == "__main__":
    # verbose("Begin of %s"%sys.argv[0])
    main()
    # verbose("End of %s"%sys.argv[0])



# -*- coding: utf-8 -*-
from Products.CPUtils.config import PROJECTNAME
from Products.PloneTestCase import PloneTestCase
from sets import Set
from Testing import ZopeTestCase

import code
import os
import sys


if __name__ == "__main__":
    execfile(os.path.join(sys.path[0], "framework.py"))

ZopeTestCase.installProduct(PROJECTNAME)

PRODUCTS = list()
PRODUCTS.append(PROJECTNAME)

testcase = PloneTestCase.PloneTestCase

# Extension profile doesn't work because appinstall of ts appears to be not run
# PloneTestCase.setupPloneSite(products=PRODUCTS, extension_profiles=EXTENSION_PROFILES)
PloneTestCase.setupPloneSite(products=PRODUCTS)


class CPUtilsTestCase(testcase):
    """Base TestCase for contacts."""

    def afterSetup(self):
        """
        Manage users and permissions
        """
        self.wft = self.portal.portal_workflow
        uf = self.portal.acl_users
        uf.userFolderAddUser("member", "member", ["Member", ], [])
        uf.userFolderAddUser("admin", "admin", ["Manager", "Member", ], [])
        uf.userFolderAddUser("anon", "anon", ["Anonymous", ], [])
        # we have to be logged as a Manager to run methods from exportimport
        self.login("admin")
        # we run every import steps
        #        self.portal.portal_setup.setImportContext("profile-Products.CPSkin2:default")
        #        self.portal.portal_setup.runImportStep(step_id="createstructure-cpskin2")
        #        self.portal.portal_setup.runImportStep(step_id="fillinleftslots-cpskin2")
        self.logout()

    def invokeFactoryC(self, type_name, id, path, **dict):
        path.invokeFactory(type_name=type_name, id=id, **dict)
        return getattr(path, id)

    def checkActionList(self, object, actions):
        """ Compare un set d'action de sorte que ['corriger', 'attendre'] soit egal a ['attendre', 'corriger'] """
        obj_actions = self.getActionList(object)
        self.assertEquals(Set(obj_actions), Set(actions))

    def getActionList(self, object):
        return [
            action_dict["name"] for action_dict in self.wft.getTransitionsFor(object)
        ]

    # Commented out for now, it gets blasted at the moment anyway.
    # Place it in the protected section if you need it.
    # def afterSetup(self):
    #    """
    #    """
    #    pass

    def interact(self, locals=None):
        """Provides an interactive shell aka console inside your testcase.

        It looks exact like in a doctestcase and you can copy and paste
        code from the shell into your doctest. The locals in the testcase are
        available, becasue you are in the testcase.

        In your testcase or doctest you can invoke the shell at any point by
        calling::

            >>> self.interact( locals() )

        locals -- passed to InteractiveInterpreter.__init__()
        """
        savestdout = sys.stdout
        sys.stdout = sys.stderr
        sys.stderr.write("=" * 70)
        console = code.InteractiveConsole(locals)
        console.interact(
            """
ZopeTestCase Interactive Console
(c) BlueDynamics Alliance, Austria - 2005

Note: You have the same locals available as in your test-case.
"""
        )
        sys.stdout.write("\nend of ZopeTestCase Interactive Console session\n")
        sys.stdout.write("=" * 70 + "\n")
        sys.stdout = savestdout


def test_suite():
    from unittest import makeSuite
    from unittest import TestSuite

    suite = TestSuite()
    suite.addTest(makeSuite(CPUtilsTestCase))
    return suite


if __name__ == "__main__":
    import framework
    framework()



#!/usr/bin/python
#

import os
import sys


buildout_inst_type = None  # True for buildout, False for manual instance
zeo_type = False  # True for zeo
was_running = True
ROTATE_CONF_FILE = "logrotate.conf"
TMPDIR = "/tmp"


def verbose(*messages):
    print ">>", " ".join(messages)


def error(*messages):
    #    print >>sys.stderr, '!!', (' '.join(messages))
    print "!!", (" ".join(messages))


# ------------------------------------------------------------------------------


def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    outfile = os.path.join(TMPDIR, "_cmd_zr.out")
    errfile = os.path.join(TMPDIR, "_cmd_zr.err")
    os.system(cmd + " >%s 2>%s" % (outfile, errfile))
    stdout = stderr = []
    try:
        if os.path.exists(outfile):
            ofile = open(outfile, "r")
            stdout = ofile.readlines()
            ofile.close()
            os.remove(outfile)
        else:
            error("File %s does not exist" % outfile)
    except IOError:
        error("Cannot open %s file" % outfile)
    try:
        if os.path.exists(errfile):
            ifile = open(errfile, "r")
            stderr = ifile.readlines()
            ifile.close()
            os.remove(errfile)
        else:
            error("File %s does not exist" % errfile)
    except IOError:
        error("Cannot open %s file" % errfile)
    return (stdout, stderr)


# ------------------------------------------------------------------------------


def stop_instance(instance_section, path):
    """ Stop the instance """
    global was_running
    if buildout_inst_type:
        cmd = ""
        if zeo_type:
            cmd = path + "/bin/zeo stop;"
        cmd += path + "/bin/%s stop" % instance_section
    else:
        cmd = path + "/bin/zopectl stop"

    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        error("\t\tError when stopping instance : '%s'" % "".join(cmd_err))
    elif cmd_out:
        cmd_out = ("".join(cmd_out)).strip("\n ")
        if (
            "daemon process not running" in cmd_out
            or "daemon manager not running" in cmd_out
        ):
            was_running = False
            error("\t\tInstance not running")
        elif "daemon process stopped" in cmd_out:
            verbose("\t\tWell stopped")
        else:
            verbose("\t\tOutput when stopping instance : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)


# ------------------------------------------------------------------------------


def start_instance(instance_section, path):
    """ Start the instance """
    if not was_running:
        return
    if buildout_inst_type:
        cmd = ""
        if zeo_type:
            cmd = path + "/bin/zeo start;"
        cmd += path + "/bin/%s start" % instance_section
    else:
        cmd = path + "/bin/zopectl start"

    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        error("\t\tError when starting instance : '%s'" % "".join(cmd_err))
    elif cmd_out:
        cmd_out = ("".join(cmd_out)).strip("\n ")
        if "daemon process started" in cmd_out:
            verbose("\t\tWell started : %s" % cmd_out)
        else:
            verbose("\t\tOutput when starting instance : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)


# ------------------------------------------------------------------------------


def rotate_logs(path):
    """
        Rotate logs of the instance
    """
    verbose("\tLog rotation on %s" % path)
    rotate_conf = os.path.join(path, ROTATE_CONF_FILE)
    if not os.path.exists(rotate_conf):
        error("\t\tRotate conf file doesn't exist '%s'" % rotate_conf)
        return
    #    cmd = '/usr/sbin/logrotate -d ' #debug
    cmd = "/usr/sbin/logrotate "
    cmd += rotate_conf

    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        error("\t\tError when running logrotate : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tOutput when running logrotate : '%s'" % "".join(cmd_out))
    else:
        pass
        # error("\t\tNo output for command : '%s'" % cmd)


# ------------------------------------------------------------------------------


def main():
    global buildout_inst_type, zeo_type
    instance_section = "instance"

    tmp = instdir
    if tmp.endswith("/"):
        tmp = tmp[0:-1]
    verbose("Working on instance %s" % tmp)

    if os.path.exists(os.path.join(tmp, "parts")):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(tmp, "etc")):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(tmp) or True:
        error("! Invalid instance path '%s' or instance type not detected" % tmp)
        sys.exit(1)

    if buildout_inst_type:
        if os.path.exists(os.path.join(tmp, "bin", "zeo")):
            zeo_type = True
            verbose("\tInstance is a zeo !")
        if os.path.exists(os.path.join(tmp, "bin", "instance1")):
            instance_section = "instance1"

    if not subcommand:
        # 1. Stop the instance if running
        stop_instance(instance_section, tmp)

        # 2. Rotate the logs of the instance
        rotate_logs(tmp)

        # 3. Restart the instance if it was running
        start_instance(instance_section, tmp)
    else:
        if buildout_inst_type:
            cmd = ""
            if zeo_type:
                cmd = tmp + "/bin/zeo %s;" % subcommand
            cmd += tmp + "/bin/%s %s" % (instance_section, subcommand)
        else:
            cmd = tmp + "/bin/zopectl %s" % subcommand

        verbose("\tRunning command '%s'" % cmd)
        (cmd_out, cmd_err) = runCommand(cmd)
        verbose("\toutput: '%s'" % "".join(cmd_out))


# ------------------------------------------------------------------------------

try:
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option(
        "-i",
        "--infos",
        dest="infos",
        default=None,
        help="infos about instance formatted like: "
        '"instance_path;transactions_days_number;admin_user;admin_password"',
    )
    parser.add_option(
        "-c",
        "--command",
        dest="subcommand",
        help="subcommand name like: start, stop, restart, status",
        default="",
    )
    (options, args) = parser.parse_args()
    if options.infos.startswith("#"):
        sys.exit(0)
    instdir, days, user, pwd = options.infos.split(";")
    subcommand = options.subcommand
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == "__main__":
    main()
